from django.dispatch import Signal
from .codes import codes

notification_received = Signal(providing_args=['notification'])
transaction_received = Signal(providing_args=['transaction'])

transaction_waiting = Signal()
transaction_analysis = Signal()
transaction_paid = Signal()
transaction_available = Signal()
transaction_dispute = Signal()
transaction_returned = Signal()
transaction_canceled = Signal()
transaction_unknown = Signal()


def dispatch_transaction(sender, **kwargs):
	transaction = kwargs.pop('transaction')
	status = int(transaction['status'])

	signals = {
		codes.waiting: transaction_waiting,
		codes.analysis: transaction_analysis,
		codes.paid: transaction_paid,
		codes.available: transaction_available,
		codes.dispute: transaction_dispute,
		codes.returned: transaction_returned,
		codes.canceled: transaction_canceled
	}

	signals.get(status, transaction_unknown).send(
		sender=None,
		transaction=transaction)

transaction_received.connect(dispatch_transaction)



class CodesDict(object):
	def __init__(self, idict={}):
		super(CodesDict, self).__setattr__('_dict', idict)

	def __setattr__(self, name, value):
		self._dict[name] = value

	def __getattr__(self, name):
		return self._dict[name]

_codes = {
	'waiting': 1,
	'analysis': 2,
	'paid': 3,
	'available': 4,
	'dispute': 5,
	'returned': 6,
	'canceled': 7,
}

codes = CodesDict(_codes)



#!/usr/bin/env python

from setuptools import setup

setup(name='djpg',
      version='0.1.0',
      author='Rafael Canovas',
      author_email='rafaelcanovas@me.com',
      url='https//github.com/rafaelcanovas/djpg',
      long_description=open('README.rst').read(),
      packages=['djpg'],
      package_data={'': ['UNLICENSE']},
      install_requires=['requests==0.14.0',
                        'furl==0.3.2',
                        'xmltodict==0.2'],
      license=open('UNLICENSE').read(),
)



from django.conf import settings
from django.http import HttpResponseRedirect

from furl import furl
import requests
import xmltodict


class Item(object):
	def __init__(self, id, description='', amount=0, quantity=1, weight=None,
				shipping_cost=None):
		self.id = id
		self.description = description
		self.amount = '%0.2f' % (amount,)
		self.quantity = quantity
		self.weight = weight
		self.shipping_cost = shipping_cost and ('%0.2f' % (shipping_cost,))


class Cart(object):
	def __init__(self, reference, currency='BRL', redirect_url=None,
				max_age=None, max_uses=None, email=None, token=None):
		self._data = {}
		self._items = []

		self._data['email'] = email or settings.PAGSEGURO_EMAIL
		self._data['token'] = token or settings.PAGSEGURO_TOKEN
		self._data['reference'] = reference
		self._data['currency'] = currency

		if redirect_url is not None:
			self._data['redirectUrl'] = redirect_url
		if max_age is not None:
			self._data['maxAge'] = max_age
		if max_uses is not None:
			self._data['maxUses'] = max_uses

		self.charset = getattr(settings, 'DEFAULT_CHARSET', 'UTF-8')
		self.checkout_url = getattr(settings, 'PAGSEGURO_CHECKOUT_URL',
				'https://ws.pagseguro.uol.com.br/v2/checkout/')
		self.payment_url = getattr(settings, 'PAGSEGURO_PAYMENT_URL',
				'https://pagseguro.uol.com.br/v2/checkout/payment.html')

	def add_item(self, item):
		if isinstance(item, Item):
			self._items.append(item)
		else:
			raise TypeError('%s expected got %s' %
							(Item.__name__, item.__class__.__name__))

	def add_items(self, *args):
		for item in args:
			self.add_item(item)

	def get_items_data(self):
		ret = {}
		for i, v in enumerate(self._items):
			o = str(i + 1)
			ret['itemId' + o] = v.id
			ret['itemDescription' + o] = v.description
			ret['itemAmount' + o] = v.amount
			ret['itemQuantity' + o] = v.quantity

			if v.weight is not None:
				ret['itemWeight' + o] = v.weight
			if v.shipping_cost is not None:
				ret['itemShippingCost' + o] = v.shipping_cost

		return ret

	def get_http_data(self):
		d = self._data.copy()
		d.update(self.get_items_data())
		return d

	def get_http_headers(self):
		return {'Content-Type':
				'application/x-www-form-urlencoded; charset=' + self.charset}

	def checkout(self):
		data = self.get_http_data()
		headers = self.get_http_headers()

		r = requests.post(self.checkout_url, data=data, headers=headers)

		if r.status_code == requests.codes.ok:
			content = xmltodict.parse(r.content)

			if content['checkout']:
				return content['checkout']['code']

	def proceed(self, code):
		return HttpResponseRedirect(furl(self.payment_url)
									.add(query_params={'code': code}).url)


class Notification(object):
	def __init__(self, type, code):
		self.type = type
		self.code = code

		self.notifications_url = getattr(settings,
			'PAGSEGURO_NOTIFICATIONS_URL',
			'https://ws.pagseguro.uol.com.br/v2/transactions/notifications')

	def fetch_content(self):
		f = furl(self.notifications_url).add(path=self.code)
		params = {
				'email': settings.PAGSEGURO_EMAIL,
				'token': settings.PAGSEGURO_TOKEN
		}

		r = requests.get(f.url, params=params)

		if r.status_code == requests.codes.ok:
			return xmltodict.parse(r.content)



from django.views.decorators.http import require_POST
from django.views.decorators.csrf import csrf_exempt
from django.http import HttpResponse, HttpResponseBadRequest

from .models import Notification
from .signals import notification_received, transaction_received


@require_POST
@csrf_exempt
def notifications(request):
	try:
		notification_type = request.POST['notificationType']
		notification_code = request.POST['notificationCode']
	except KeyError:
		return HttpResponseBadRequest()

	notification = Notification(type=notification_type, code=notification_code)
	notification_received.send(sender=None, notification=notification)

	content = notification.fetch_content()
	if content:
		if notification.type == 'transaction':
			try:
				transaction = content['transaction']
				transaction_received.send(sender=None, transaction=transaction)
			except KeyError:
				pass

	return HttpResponse()
